@@ -7,6 +7,7 @@ extern const struct imv_backend imv_backend_libpng;
 extern const struct imv_backend imv_backend_librsvg;
 extern const struct imv_backend imv_backend_libtiff;
 extern const struct imv_backend imv_backend_libjpeg;
+extern const struct imv_backend imv_backend_libnsgif;
 
 int main(int argc, char **argv)
 {
@@ -32,6 +33,10 @@ int main(int argc, char **argv)
   imv_install_backend(imv, &imv_backend_librsvg);
 #endif
 
+#ifdef IMV_BACKEND_LIBNSGIF
+  imv_install_backend(imv, &imv_backend_libnsgif);
+#endif
+
 #ifdef IMV_BACKEND_FREEIMAGE
   imv_install_backend(imv, &imv_backend_freeimage);
 #endif
