@@ -7,13 +7,10 @@
 #include <fcntl.h>
 #include <limits.h>
 #include <poll.h>
-#include <pthread.h>
 #include <signal.h>
-#include <stdlib.h>
 #include <string.h>
 #include <sys/mman.h>
 #include <time.h>
-#include <unistd.h>
 
 #include <wayland-client.h>
 #include <wayland-egl.h>
